@@ -1,7 +1,7 @@
 /*
  *  Copyright (C) 2001,2002,2003 Nikos Mavroyanopoulos
  *
- * This file is part of GNUTLS.
+ *  This file is part of GNUTLS.
  *
  *  The GNUTLS library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public   
@@ -42,6 +42,89 @@
 #include "x509/x509.h"
 #include "x509/mpi.h"
 
+/**
+  * gnutls_certificate_free_keys - Used to free all the keys from a gnutls_certificate_credentials structure
+  * @sc: is an &gnutls_certificate_credentials structure.
+  *
+  * This function will delete all the keys and the certificates associated
+  * with the given credentials. This function must not be called when a
+  * TLS negotiation that uses the credentials is in progress.
+  *
+  **/
+void gnutls_certificate_free_keys(gnutls_certificate_credentials sc)
+{
+	uint i, j;
+
+	for (i = 0; i < sc->ncerts; i++) {
+		for (j = 0; j < sc->cert_list_length[i]; j++) {
+			_gnutls_free_cert( &sc->cert_list[i][j]);
+		}
+		gnutls_free( sc->cert_list[i]);
+	}
+
+	gnutls_free(sc->cert_list_length);
+	sc->cert_list_length = NULL;
+
+	gnutls_free(sc->cert_list);
+	sc->cert_list = NULL;
+
+	for (i = 0; i < sc->ncerts; i++) {
+		_gnutls_privkey_deinit( &sc->pkey[i]);
+	}
+
+	gnutls_free( sc->pkey);
+	sc->pkey = NULL;
+	
+	sc->ncerts = 0;
+
+}
+
+/**
+  * gnutls_certificate_free_cas - Used to free all the CAs from a gnutls_certificate_credentials structure
+  * @sc: is an &gnutls_certificate_credentials structure.
+  *
+  * This function will delete all the CAs associated
+  * with the given credentials.
+  *
+  **/
+void gnutls_certificate_free_cas(gnutls_certificate_credentials sc)
+{
+	uint j;
+
+        for (j = 0; j < sc->x509_ncas; j++) {
+		gnutls_x509_crt_deinit( sc->x509_ca_list[j]);
+        }
+        
+        sc->x509_ncas = 0;
+
+	gnutls_free( sc->x509_ca_list);
+	sc->x509_ca_list = NULL;
+
+	_gnutls_free_datum( &sc->x509_rdn_sequence);
+
+}
+
+/**
+  * gnutls_certificate_free_crls - Used to free all the CRLs from a gnutls_certificate_credentials structure
+  * @sc: is an &gnutls_certificate_credentials structure.
+  *
+  * This function will delete all the CRLs associated
+  * with the given credentials.
+  *
+  **/
+void gnutls_certificate_free_crls(gnutls_certificate_credentials sc)
+{
+	uint j;
+
+        for (j = 0; j < sc->x509_ncrls; j++) {
+		gnutls_x509_crl_deinit( sc->x509_crl_list[j]);
+        }
+        
+        sc->x509_ncrls = 0;
+
+	gnutls_free( sc->x509_crl_list);
+	sc->x509_crl_list = NULL;
+}
 
 /**
   * gnutls_certificate_free_credentials - Used to free an allocated gnutls_certificate_credentials structure
@@ -57,36 +140,12 @@
   **/
 void gnutls_certificate_free_credentials(gnutls_certificate_credentials sc)
 {
-	uint i, j;
+	gnutls_certificate_free_keys( sc);
+	gnutls_certificate_free_cas( sc);
+	gnutls_certificate_free_crls( sc);
 
-	for (i = 0; i < sc->ncerts; i++) {
-		for (j = 0; j < sc->cert_list_length[i]; j++) {
-			_gnutls_free_cert( &sc->cert_list[i][j]);
-		}
-		gnutls_free( sc->cert_list[i]);
-	}
-
-	gnutls_free(sc->cert_list_length);
-	gnutls_free(sc->cert_list);
-
-	for (j = 0; j < sc->x509_ncas; j++) {
-		gnutls_x509_crt_deinit( sc->x509_ca_list[j]);
-	}
-
-	for (j = 0; j < sc->x509_ncrls; j++) {
-		gnutls_x509_crl_deinit( sc->x509_crl_list[j]);
-	}
-
-	gnutls_free( sc->x509_ca_list);
 	_gnutls_free_datum( &sc->keyring);
 
-	for (i = 0; i < sc->ncerts; i++) {
-		_gnutls_privkey_deinit( &sc->pkey[i]);
-	}
-
-	gnutls_free( sc->pkey);
-	gnutls_free( sc->x509_rdn_sequence.data);
-
 	gnutls_free( sc);
 }
 
@@ -457,15 +516,46 @@ int _gnutls_x509_cert2gnutls_cert(gnutls_cert * gcert, const gnutls_datum *derCe
 		return ret;
 	}
 	
+	ret = _gnutls_x509_crt2gnutls_cert( gcert, cert, flags);
+	gnutls_x509_crt_deinit( cert);
+	
+	return ret;
+}
+
+/* Like above but it accepts a parsed certificate instead.
+ */
+int _gnutls_x509_crt2gnutls_cert(gnutls_cert * gcert, gnutls_x509_crt cert,
+	unsigned int flags)
+{
+	int ret = 0;
+	
 	memset(gcert, 0, sizeof(gnutls_cert));
 	gcert->cert_type = GNUTLS_CRT_X509;
 
 	if ( !(flags & CERT_NO_COPY)) {
-		if (_gnutls_set_datum(&gcert->raw, derCert->data, derCert->size) < 0) {
+		opaque* der;
+		int der_size = 0;
+		
+		ret = gnutls_x509_crt_export( cert, GNUTLS_X509_FMT_DER, NULL, &der_size);
+		if (ret != GNUTLS_E_SHORT_MEMORY_BUFFER) {
+			gnutls_assert();
+			return ret;
+		}
+		
+		der = gnutls_malloc( der_size);
+		if (der == NULL) {
 			gnutls_assert();
-			gnutls_x509_crt_deinit( cert);
 			return GNUTLS_E_MEMORY_ERROR;
 		}
+
+		ret = gnutls_x509_crt_export( cert, GNUTLS_X509_FMT_DER, der, &der_size);
+		if (ret < 0) {
+			gnutls_assert();
+			return ret;
+		}
+
+		gcert->raw.data = der;
+		gcert->raw.size = der_size;
 	} else
 		/* now we have 0 or a bitwise or of things to decode */
 		flags ^= CERT_NO_COPY;
@@ -482,13 +572,10 @@ int _gnutls_x509_cert2gnutls_cert(gnutls_cert * gcert, const gnutls_datum *derCe
 		ret = _gnutls_x509_crt_get_mpis( cert, gcert->params, &gcert->params_size);
 		if (ret < 0) {
 			gnutls_assert();
-			gnutls_x509_crt_deinit( cert);
 			return ret;
 		}
 	}
 
-	gnutls_x509_crt_deinit( cert);
-
 	return 0;
 
 }
