@@ -242,7 +242,7 @@ int _gnutls_x509_parse_dn(ASN1_TYPE asn1_struct,
 								  &sizeof_string))
 				    < 0) {
 					gnutls_assert();
-					_gnutls_log("Found OID: '%s' with value '%s'\n", 
+					_gnutls_x509_log("Found OID: '%s' with value '%s'\n", 
 						oid, _gnutls_bin2hex(value, len, escaped, sizeof(escaped)) );
 					goto cleanup;
 				}
@@ -488,7 +488,8 @@ int _gnutls_x509_parse_dn_oid(ASN1_TYPE asn1_struct,
 int _gnutls_x509_encode_and_write_attribute( const char* given_oid, ASN1_TYPE asn1_struct, 
 	const char* where, const unsigned char* data, int sizeof_data, int multi) 
 {
-char val_name[MAX_NAME_SIZE], tmp[128];
+const char *val_name;
+char tmp[128];
 ASN1_TYPE c2;
 opaque *der;
 int der_len, result;
@@ -496,10 +497,10 @@ int der_len, result;
 
 	/* Find how to encode the data.
 	 */
-	result = asn1_find_structure_from_oid( _gnutls_get_pkix(), given_oid, val_name);
-	if (result != ASN1_SUCCESS) {
+	val_name = asn1_find_structure_from_oid( _gnutls_get_pkix(), given_oid);
+	if (val_name == NULL) {
 		gnutls_assert();
-		return _gnutls_asn2err(result);
+		return GNUTLS_E_ASN1_GENERIC_ERROR;
 	}
 
 	_gnutls_str_cpy( tmp, sizeof(tmp), "PKIX1.");
@@ -549,9 +550,10 @@ int der_len, result;
 		return _gnutls_asn2err(result);
 	}
 
-	der_len = sizeof_data + 64;
-	der = gnutls_alloca( der_len);
+	der_len = 0;
+	asn1_der_coding( c2, "", NULL, &der_len, NULL);
 
+	der = gnutls_alloca( der_len);
 	result = asn1_der_coding( c2, "", der, &der_len, NULL);
 	
 	asn1_delete_structure( &c2);
