@@ -68,8 +68,6 @@ static int decrypt_data( ASN1_TYPE pkcs8_asn, char* password,
 static ASN1_TYPE decode_private_key_info( const gnutls_datum* der, gnutls_x509_privkey pkey);
 static int write_pbe2_params( ASN1_TYPE pkcs8_asn, const struct pbkdf2_params* kdf_params,
 	const struct pbe_enc_params* enc_params);
-static int _der_encode_and_copy( ASN1_TYPE src, const char* src_name,
-	ASN1_TYPE dest, const char* dest_name);
 static int encrypt_data( const gnutls_datum* plain, 
 	const struct pbe_enc_params *enc_params, 
 	gnutls_datum* key, gnutls_datum* encrypted);
@@ -291,6 +289,7 @@ static ASN1_TYPE encode_to_pkcs8_key( const gnutls_datum *raw_key,
   * @output_data_size: holds the size of output_data (and will be replaced by the actual size of parameters)
   *
   * This function will export the private key to a PKCS8 structure.
+  * Currently only RSA keys can be exported.
   *
   * If the buffer provided is not long enough to hold the output, then
   * GNUTLS_E_SHORT_MEMORY_BUFFER will be returned.
@@ -564,6 +563,7 @@ static ASN1_TYPE decode_private_key_info( const gnutls_datum* der, gnutls_x509_p
   *
   * This function will convert the given DER or PEM encoded PKCS8 2.0 encrypted key
   * to the native gnutls_x509_privkey format. The output will be stored in 'key'.
+  * Currently only RSA keys can be imported.
   *
   * If the Certificate is PEM encoded it should have a header of "ENCRYPTED PRIVATE KEY",
   * or "PRIVATE KEY". You only need to specify the flags if the key is DER encoded.
@@ -894,52 +894,6 @@ int key_size;
 }
 
 
-
-/* DER Encodes the src ASN1_TYPE and stores it to
- * dest in dest_name. Usefull to encode something and store it
- * as OCTET.
- */
-static int _der_encode_and_copy( ASN1_TYPE src, const char* src_name,
-	ASN1_TYPE dest, const char* dest_name)
-{
-int size, result;
-opaque *data = NULL;
-
-	size = 0;
-	result = asn1_der_coding( src, src_name, NULL, &size, NULL);
-	if (result != ASN1_MEM_ERROR) {
-		gnutls_assert();
-		return _gnutls_asn2err(result);
-	}
-
-	/* allocate data for the der
-	 */
-	data = gnutls_alloca( size);
-	if (data == NULL) {
-		gnutls_assert();
-		return GNUTLS_E_MEMORY_ERROR;
-	}
-
-	result = asn1_der_coding( src, src_name, data, &size, NULL);
-	if (result != ASN1_SUCCESS) {
-		gnutls_assert();
-		gnutls_afree(data);
-		return _gnutls_asn2err(result);
-	}
-	
-	/* Write the key derivation algorithm
-	 */
-	result = asn1_write_value( dest, dest_name, data, size);
-	if (result != ASN1_SUCCESS) {
-		gnutls_assert();
-		gnutls_afree(data);
-		return _gnutls_asn2err(result);
-	}
-
-	return 0;
-}
-
-
 /* Writes the PBKDF2 parameters.
  */
 static int write_pbkdf2_params( ASN1_TYPE pbes2_asn, const struct pbkdf2_params *kdf_params)
@@ -1017,7 +971,7 @@ char tmp[64];
 	/* now encode them an put the DER output
 	 * in the keyDerivationFunc.parameters
 	 */
-	result = _der_encode_and_copy( pbkdf2_asn, "", 
+	result = _gnutls_x509_der_encode_and_copy( pbkdf2_asn, "", 
 		pbes2_asn, "keyDerivationFunc.parameters");
 	if (result < 0) {
 		gnutls_assert();
@@ -1068,7 +1022,7 @@ ASN1_TYPE pbe_asn = ASN1_TYPE_EMPTY;
 	/* now encode them an put the DER output
 	 * in the encryptionScheme.parameters
 	 */
-	result = _der_encode_and_copy( pbe_asn, "", 
+	result = _gnutls_x509_der_encode_and_copy( pbe_asn, "", 
 		pbes2_asn, "encryptionScheme.parameters");
 	if (result < 0) {
 		gnutls_assert();
@@ -1157,7 +1111,7 @@ ASN1_TYPE pbes2_asn = ASN1_TYPE_EMPTY;
 		goto error;
 	}
 
-	result = _der_encode_and_copy( pbes2_asn, "", 
+	result = _gnutls_x509_der_encode_and_copy( pbes2_asn, "", 
 		pkcs8_asn, "encryptionAlgorithm.parameters");
 	if (result < 0) {
 		gnutls_assert();
