@@ -224,7 +224,7 @@ int gnutls_x509_crt_import(gnutls_x509_crt cert, const gnutls_datum * data,
 		
 		if (len % 8 != 0) {
 			gnutls_assert();
-			result = GNUTLS_E_UNIMPLEMENTED_FEATURE;
+			result = GNUTLS_E_CERTIFICATE_ERROR;
 			goto cleanup;
 		}
 		
@@ -1068,7 +1068,7 @@ gnutls_datum tmp;
   * gnutls_x509_crt_export - This function will export the certificate
   * @cert: Holds the certificate
   * @format: the format of output params. One of PEM or DER.
-  * @output_data: will contain a private key PEM or DER encoded
+  * @output_data: will contain a certificate PEM or DER encoded
   * @output_data_size: holds the size of output_data (and will be replaced by the actual size of parameters)
   *
   * This function will export the certificate to DER or PEM format.
@@ -1091,75 +1091,3 @@ int gnutls_x509_crt_export( gnutls_x509_crt cert,
 }
 
 
-/* A generic export function. Will export the given ASN.1 encoded data
- * to PEM or DER raw data.
- */
-int _gnutls_x509_export_int( ASN1_TYPE asn1_data,
-	gnutls_x509_crt_fmt format, char* pem_header,
-	int tmp_buf_size, unsigned char* output_data, int* output_data_size)
-{
-	int result;
-	if (tmp_buf_size == 0) tmp_buf_size = 16*1024;
-	
-	if (format == GNUTLS_X509_FMT_DER) {
-		if (output_data == NULL) *output_data_size = 0;
-	
-		if ((result=asn1_der_coding( asn1_data, "", output_data, output_data_size, NULL)) != ASN1_SUCCESS) {
-			if (result == ASN1_MEM_ERROR)
-				return GNUTLS_E_SHORT_MEMORY_BUFFER;
-
-			gnutls_assert();
-			return _gnutls_asn2err(result);
-		}
-
-	} else { /* PEM */
-		opaque *tmp;
-		opaque *out;
-		int len = tmp_buf_size;
-		
-		tmp = gnutls_alloca( len);
-		if (tmp == NULL) {
-			gnutls_assert();
-			return GNUTLS_E_MEMORY_ERROR;
-		}
-
-		if ((result=asn1_der_coding( asn1_data, "", tmp, &len, NULL)) != ASN1_SUCCESS) {
-			gnutls_assert();
-			gnutls_afree(tmp);
-			return _gnutls_asn2err(result);
-		}
-
-		result = _gnutls_fbase64_encode( pem_header,
-						tmp, len, &out);
-
-		gnutls_afree(tmp);
-
-		if (result < 0) {
-			gnutls_assert();
-			return result;
-		}
-
-		if (result == 0) {	/* oooops */
-			gnutls_assert();
-			return GNUTLS_E_INTERNAL_ERROR;
-		}
-
-		if (result + 1 > *output_data_size) {
-			gnutls_assert();
-			gnutls_free(out);
-			*output_data_size = result;
-			return GNUTLS_E_SHORT_MEMORY_BUFFER;
-		}
-
-		*output_data_size = result;
-		
-		if (output_data) {
-			memcpy( output_data, out, result);
-			output_data[result] = 0;
-		}
-		gnutls_free( out);
-		
-	}
-
-	return 0;
-}
